#ifndef _OS_H_
#define _OS_H_



/*
 * Purpose: OS specific definitions for Minix
 *
 */
/*
 *
 * This file is part of Open Sound System.
 *
 * Copyright (C) 4Front Technologies 1996-2008.
 *
 * This this source file is released under GPL v2 license (no other versions).
 * See the COPYING file included in the main directory of this source
 * distribution for the license terms and conditions.
 *
 */





#include <sys/types.h>
#include <stdlib.h>
#include <alloca.h>

#include <oss_errno.h>
#include <stddef.h>
#include <string.h>
#include <fcntl.h>
#include <stdint.h>

#include <timers.h>

#include "pci_helper.h"
#include "kernel/OS/Minix/minixdefs.h"



#undef OS_VERSION
#define OS_VERSION 3.1.4
#define MINIX
#define MAIN_DRIVER "oss"




/* Memory Mapped devices */
#define PCI_READL(osp, p)            (*(volatile u32_t *) (p))
#define PCI_READW(osp, p)            (*(volatile u16_t *) (p))
#define PCI_READB(osp, p)            (*(volatile u8_t *) (p))
#define PCI_WRITEL(osp, addr, data)  (*(volatile u32_t *) (addr) = (data))
#define PCI_WRITEW(osp, addr, data)  (*(volatile u16_t *) (addr) = (data))
#define PCI_WRITEB(osp, addr, data)  (*(volatile u8_t *) (addr) = (data))



extern int detect_trace;
#define DDB(x) if (detect_trace) x


/*#define USE_DEVICE_SUBDIRS*/


typedef void *oss_dma_handle_t;


#undef HZ
extern int oss_hz;
#define OSS_HZ oss_hz



#include "soundcard.h"




typedef int _oss_mutex_t;
typedef _oss_mutex_t *oss_mutex_t;


/* TODO */
/*#define ALLOW_BUFFER_MAPPING*/


struct fileinfo {
	int mode;
	int acc_flags;
	int pid;
	int dev;
	char *cmd;
};


#define ISSET_FILE_FLAG(fileinfo, flag)  (fileinfo->acc_flags & (flag) ? 1:0)




void *oss_contig_malloc(unsigned long sz, unsigned long memlimit, 
		oss_native_word *phaddr);
void oss_contig_free(void *p, unsigned long sz);
#define CONTIG_MALLOC(osdev, sz, memlimit, phaddr, handle)  oss_contig_malloc(sz, memlimit, phaddr)
#define CONTIG_FREE(osdev, p, sz, handle)   oss_contig_free(p, sz)








struct _oss_pll_event_t {
	short events, revents;
};
typedef struct _oss_poll_event_t oss_poll_event_t;



#undef timeout
#define timeout oss_timeout
#undef untimeout
#define untimeout oss_untimeout
typedef int timeout_id_t;
#ifdef OSS_NO_LONG_LONG
extern timeout_id_t oss_timeout (void (*func) (void *), void *arg,
		unsigned long ticks);
#else
extern timeout_id_t oss_timeout (void (*func) (void *), void *arg,
		unsigned long long ticks);
#endif
extern void oss_untimeout (timeout_id_t id);




#define MUTEX_ENTER_IRQDISABLE(mutex, flags)  
#define MUTEX_ENTER(mutex, flags) 
#define MUTEX_EXIT(mutex, flags) 


#define INB(o, p) pci_inb(p)
#define INW(o, p) pci_inw(p)
#define INL(o, p) pci_inl(p)

#define OUTB(o, v, p) pci_outb(p, v)
#define OUTW(o, v, p) pci_outw(p, v)
#define OUTL(o, v, p) pci_outl(p, v)

#define MUTEX_EXIT_IRQRESTORE(mutex, flags)


int cmn_err(int level, const char *format, ...);





#ifdef _KERNEL
struct oss_wait_queue
{
	oss_mutex_t mutex;
	unsigned long flags;
	int wakeup;
	int expired;
	timer_t timer;
};
#endif



void *oss_map_pci_mem (oss_device_t *osdev, int nr, oss_native_word paddr, int psize); 

void oss_unmap_pci_mem (oss_device_t *osdev, int nr, oss_native_word paddr, void *virt, int psize);

#define MAP_PCI_IOADDR(osdev, nr, io) (oss_native_word)(io)
#define UNMAP_PCI_IOADDR(osdev, ix) {}

#define MAP_PCI_MEM(osdev, ix, phaddr, size) oss_map_pci_mem(osdev, ix, phaddr, size)
#define UNMAP_PCI_MEM(osdev, ix, ph, virt, size) {} 


#define MUTEX_INIT(osdev, mutex, hier)
#define MUTEX_CLEANUP(mutex) 



extern void oss_udelay (unsigned long d);
extern int sprintf (char *buf, const char *s, ...);
extern oss_mutex_t oss_mutex_init (void);
extern void oss_mutex_cleanup (oss_mutex_t mutex);



#define CE_CONT     0
#define CE_NOTE     1
#define CE_WARN     2
#define CE_PANIC    3


#define GET_PROCESS_NAME(f) (char *)f
#define GET_PROCESS_PID(f) (int)f


#ifdef OSS_NO_LONG_LONG
typedef long oss_int64_t;
typedef long oss_uint64_t;
#else
typedef long long oss_int64_t;
typedef unsigned long long oss_uint64_t;
#endif






#define uiomove oss_uiomove


extern int oss_uiomove (void *address, size_t nbytes, enum uio_rw rwflag, 
		uio_t *uio_p);


extern unsigned long oss_get_time(void);
#define GET_JIFFIES() oss_get_time()



#define POLLIN 0
#define POLLRDNORM 1
#define POLLOUT 2
#define POLLWRNORM 3




#define KERNEL_MALLOC(nbytes)   alloc_mem(nbytes)
#define KERNEL_FREE(addr)   {if (addr)free_mem(addr);addr=NULL;}

void *alloc_mem(int nbytes);
void free_mem(void *addr);



#if 1
#define __inline__ /* */
#endif

#endif
